@@ -42,6 +42,7 @@ import org.apache.maven.cli.internal.extension.model.io.xpp3.CoreExtensionsXpp3R
 import org.codehaus.plexus.util.StringUtils;
 import org.codehaus.plexus.util.xml.pull.XmlPullParserException;
 import org.mvndaemon.mvnd.common.Environment;
+import org.mvndaemon.mvnd.common.InterpolationHelper;
 import org.mvndaemon.mvnd.common.Os;
 import org.mvndaemon.mvnd.common.SocketFamily;
 import org.mvndaemon.mvnd.common.TimeUtils;
@@ -58,6 +59,7 @@ public class DaemonParameters {
     private static final Logger LOG = LoggerFactory.getLogger(DaemonParameters.class);
     private static final String EXT_CLASS_PATH = "maven.ext.class.path";
     private static final String EXTENSIONS_FILENAME = ".mvn/extensions.xml";
+    private static final String ENV_PREFIX = "env.";
 
     protected final Map<Path, Properties> mvndProperties = new ConcurrentHashMap<>();
     protected final Function<Path, Properties> provider = path -> mvndProperties.computeIfAbsent(path,
@@ -103,10 +105,10 @@ public class DaemonParameters {
                         description -> description.append("path relative to the mvnd executable"),
                         this::mvndHomeFromExecutable))
                 .orSystemProperty()
-                .orEnvironmentVariable()
                 .orLocalProperty(provider, suppliedPropertiesPath())
                 .orLocalProperty(provider, localPropertiesPath())
                 .orLocalProperty(provider, userPropertiesPath())
+                .orEnvironmentVariable()
                 .orFail()
                 .asPath()
                 .toAbsolutePath().normalize();
@@ -129,12 +131,12 @@ public class DaemonParameters {
 
     public Path javaHome() {
         final Path result = value(Environment.JAVA_HOME)
-                .orEnvironmentVariable()
                 .orLocalProperty(provider, suppliedPropertiesPath())
                 .orLocalProperty(provider, localPropertiesPath())
                 .orLocalProperty(provider, userPropertiesPath())
                 .orLocalProperty(provider, globalPropertiesPath())
                 .orSystemProperty()
+                .orEnvironmentVariable()
                 .orFail()
                 .asPath();
         try {
@@ -162,8 +164,8 @@ public class DaemonParameters {
 
     public Path suppliedPropertiesPath() {
         return value(Environment.MVND_PROPERTIES_PATH)
-                .orEnvironmentVariable()
                 .orSystemProperty()
+                .orEnvironmentVariable()
                 .asPath();
     }
 
@@ -191,6 +193,8 @@ public class DaemonParameters {
     public Path daemonStorage() {
         return value(Environment.MVND_DAEMON_STORAGE)
                 .orSystemProperty()
+                .orLocalProperty(provider, globalPropertiesPath())
+                .orEnvironmentVariable()
                 .orDefault(
                         () -> userHome().resolve(".m2/mvnd/registry/" + BuildProperties.getInstance().getVersion()).toString())
                 .asPath();
@@ -451,6 +455,10 @@ public class DaemonParameters {
         if (Files.exists(path)) {
             try (InputStream in = Files.newInputStream(path)) {
                 result.load(in);
+                Properties sysProps = new Properties();
+                sysProps.putAll(System.getProperties());
+                System.getenv().forEach((k, v) -> sysProps.put(k, ENV_PREFIX + v));
+                InterpolationHelper.performSubstitution(result, sysProps::getProperty, true, true);
             } catch (IOException e) {
                 throw new RuntimeException("Could not read " + path);
             }
